# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .location import Location
from ..._models import BaseModel

__all__ = [
    "DriverAssignmentAssignResponse",
    "Result",
    "ResultAlternateAssignment",
    "ResultAlternateAssignmentAssignment",
    "ResultTrip",
    "ResultTripVehicle",
    "ResultTripVehicleSteps",
    "ResultUnassignedOrder",
]


class ResultAlternateAssignmentAssignment(BaseModel):
    pickup_eta: Optional[int] = None
    """Returns the ETA to the order's pickup location for the given vehicle."""

    vehicle_id: Optional[str] = None
    """Returns the vehicle ID which could potentially be assigned to the given order."""


class ResultAlternateAssignment(BaseModel):
    assignments: Optional[List[ResultAlternateAssignmentAssignment]] = None
    """An array of objects containing the details of the alternate vehicle assignments.

    Each object represents an alternate vehicle assignment.
    """

    order_id: Optional[str] = None
    """Returns the order ID associated with the alternate assignments."""


class ResultTripVehicleSteps(BaseModel):
    distance: Optional[int] = None
    """
    Returns the driving distance, in meters, to the step's location from previous
    step's location. For the first step of a trip, `distance` indicates the driving
    distance from `vehicle_current_location` to the step's location.
    """

    eta: Optional[int] = None
    """
    Returns the driving duration, in seconds, to the step's location from previous
    step's location. For the first step of a trip, `eta` indicates the driving
    duration from `vehicle_current_location` to the step's location.
    """

    location: Optional[Location] = None
    """Location info."""

    order_id: Optional[str] = None
    """Returns the ID of the order.

    In case the step type is **ongoing**, an empty string is returned.
    """

    type: Optional[Literal["pickup", "dropoff", "ongoing"]] = None
    """Returns the type of the step. Currently, it can take following values:

    - **pickup:** Indicates the pickup step for an order
    - **dropoff:** Indicates the dropoff step for an order. It is returned only if
      `dropoff_details` was **true** in the input request.
    - **ongoing:** Indicates a step that the vehicle needs to complete on its
      current trip. This is returned in the response only when `remaining_waypoints`
      input was provided for the given vehicle.
    - **intermediate_waypoint:** Indicates an intermediate stop that the vehicle
      needs to complete in case multiple dropoffs are provided in the input.
    """


class ResultTripVehicle(BaseModel):
    id: Optional[str] = None
    """Returns the ID of the vehicle."""

    steps: Optional[ResultTripVehicleSteps] = None
    """
    A collection of objects returning the sequence of steps that the vehicle needs
    to perform for a trip.
    """

    vehicle_current_location: Optional[Location] = None
    """Location info."""


class ResultTrip(BaseModel):
    trip_id: Optional[str] = None
    """Returns a unique trip ID."""

    vehicle: Optional[ResultTripVehicle] = None
    """Returns the details of the vehicle, assigned order and the trip steps."""


class ResultUnassignedOrder(BaseModel):
    order_id: Optional[str] = None
    """Returns the ID of the order which remained unassigned."""

    unassigned_reason: Optional[str] = None
    """Returns the most probable reason due to which the order remained unassigned."""


class Result(BaseModel):
    alternate_assignments: Optional[List[ResultAlternateAssignment]] = None
    """
    An array of objects containing the details of the potential, alternate vehicle
    assignments for the orders in the input. This attribute will not be returned in
    the response if the `alternate_assignments` was not provided in the input. Each
    object represents alternate assignments for a single order.
    """

    available_vehicles: Optional[List[str]] = None
    """A collection of vehicles IDs that were not assigned to any orders.

    A `null` value is returned if there are no vehicles without an order assignment.
    """

    trips: Optional[List[ResultTrip]] = None
    """
    An collection of objects returning the trip details for each vehicle which was
    assigned to an order. Each object corresponds to one vehicle.
    """

    unassigned_orders: Optional[List[ResultUnassignedOrder]] = None
    """A collection of objects listing the details of orders which remained unassigned.

    Each object represents a single order. A `null` value is returned if there are
    no unassigned orders.
    """


class DriverAssignmentAssignResponse(BaseModel):
    message: Optional[str] = None
    """Displays indicative error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    result: Optional[Result] = None
    """An object containing the details of the assignments."""

    status: Optional[int] = None
    """An integer indicating the HTTP response code.

    See the
    [API Error Handling](https://docs.nextbillion.ai/optimization/driver-assignment-api#api-error-handling)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.optimization import DriverAssignmentAssignResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDriverAssignment:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_assign(self, client: NextbillionSDK) -> None:
        driver_assignment = client.optimization.driver_assignment.assign(
            key="key=API_KEY",
            filter={},
            orders=[
                {
                    "id": "id",
                    "pickup": {},
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )
        assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_assign_with_all_params(self, client: NextbillionSDK) -> None:
        driver_assignment = client.optimization.driver_assignment.assign(
            key="key=API_KEY",
            filter={
                "driving_distance": 0,
                "pickup_eta": 0,
                "radius": 0,
            },
            orders=[
                {
                    "id": "id",
                    "pickup": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": {},
                    "dropoffs": [
                        {
                            "lat": 0,
                            "lng": 0,
                        }
                    ],
                    "priority": 0,
                    "service_time": 0,
                    "vehicle_preferences": {
                        "exclude_all_of_attributes": [
                            {
                                "attribute": '"attribute": "driver_rating"',
                                "operator": '"operator":"<"',
                                "value": '"value": "4"',
                            }
                        ],
                        "required_all_of_attributes": [
                            {
                                "attribute": '"attribute": "driver_rating"',
                                "operator": '"operator":"=="',
                                "value": '"value": "4"',
                            }
                        ],
                        "required_any_of_attributes": [
                            {
                                "attribute": '"attribute": "driver_rating"',
                                "operator": '"operator":">"',
                                "value": '"value": "4"',
                            }
                        ],
                    },
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                    "priority": 0,
                    "remaining_waypoints": [
                        {
                            "lat": -90,
                            "lon": -180,
                        }
                    ],
                }
            ],
            options={
                "alternate_assignments": 0,
                "dropoff_details": True,
                "order_attribute_priority_mappings": [
                    {
                        "attribute": '"attribute": "driver_rating"',
                        "operator": '"operator":"=="',
                        "priority": "priority",
                        "value": '"value": "4"',
                    }
                ],
                "travel_cost": "driving_eta",
                "vehicle_attribute_priority_mappings": [
                    {
                        "attribute": '"attribute": "driver_rating"',
                        "operator": '"operator":"=="',
                        "priority": "priority",
                        "value": '"value": "4"',
                    }
                ],
            },
        )
        assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_assign(self, client: NextbillionSDK) -> None:
        response = client.optimization.driver_assignment.with_raw_response.assign(
            key="key=API_KEY",
            filter={},
            orders=[
                {
                    "id": "id",
                    "pickup": {},
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        driver_assignment = response.parse()
        assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_assign(self, client: NextbillionSDK) -> None:
        with client.optimization.driver_assignment.with_streaming_response.assign(
            key="key=API_KEY",
            filter={},
            orders=[
                {
                    "id": "id",
                    "pickup": {},
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            driver_assignment = response.parse()
            assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDriverAssignment:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_assign(self, async_client: AsyncNextbillionSDK) -> None:
        driver_assignment = await async_client.optimization.driver_assignment.assign(
            key="key=API_KEY",
            filter={},
            orders=[
                {
                    "id": "id",
                    "pickup": {},
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )
        assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_assign_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        driver_assignment = await async_client.optimization.driver_assignment.assign(
            key="key=API_KEY",
            filter={
                "driving_distance": 0,
                "pickup_eta": 0,
                "radius": 0,
            },
            orders=[
                {
                    "id": "id",
                    "pickup": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": {},
                    "dropoffs": [
                        {
                            "lat": 0,
                            "lng": 0,
                        }
                    ],
                    "priority": 0,
                    "service_time": 0,
                    "vehicle_preferences": {
                        "exclude_all_of_attributes": [
                            {
                                "attribute": '"attribute": "driver_rating"',
                                "operator": '"operator":"<"',
                                "value": '"value": "4"',
                            }
                        ],
                        "required_all_of_attributes": [
                            {
                                "attribute": '"attribute": "driver_rating"',
                                "operator": '"operator":"=="',
                                "value": '"value": "4"',
                            }
                        ],
                        "required_any_of_attributes": [
                            {
                                "attribute": '"attribute": "driver_rating"',
                                "operator": '"operator":">"',
                                "value": '"value": "4"',
                            }
                        ],
                    },
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                    "priority": 0,
                    "remaining_waypoints": [
                        {
                            "lat": -90,
                            "lon": -180,
                        }
                    ],
                }
            ],
            options={
                "alternate_assignments": 0,
                "dropoff_details": True,
                "order_attribute_priority_mappings": [
                    {
                        "attribute": '"attribute": "driver_rating"',
                        "operator": '"operator":"=="',
                        "priority": "priority",
                        "value": '"value": "4"',
                    }
                ],
                "travel_cost": "driving_eta",
                "vehicle_attribute_priority_mappings": [
                    {
                        "attribute": '"attribute": "driver_rating"',
                        "operator": '"operator":"=="',
                        "priority": "priority",
                        "value": '"value": "4"',
                    }
                ],
            },
        )
        assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_assign(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.optimization.driver_assignment.with_raw_response.assign(
            key="key=API_KEY",
            filter={},
            orders=[
                {
                    "id": "id",
                    "pickup": {},
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        driver_assignment = await response.parse()
        assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_assign(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.optimization.driver_assignment.with_streaming_response.assign(
            key="key=API_KEY",
            filter={},
            orders=[
                {
                    "id": "id",
                    "pickup": {},
                }
            ],
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            driver_assignment = await response.parse()
            assert_matches_type(DriverAssignmentAssignResponse, driver_assignment, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["NamespacedApikeyDeleteResponse"]


class NamespacedApikeyDeleteResponse(BaseModel):
    msg: Optional[str] = None
    """Its value is `OK` in case of a successful delete operation.

    Indicative error messages are returned otherwise, for different errors.
    """

    status: Optional[int] = None
    """A string indicating the state of the response.

    A successful delete operation ins indicated by an HTTP code of`200`. See the
    [API Error Codes](https://docs.nextbillion.ai/docs/tracking/api/live-tracking-api#api-error-codes)
    section below for possible values in case of errors.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel
from .route_step_completion_mode import RouteStepCompletionMode
from .route_step_geofence_config import RouteStepGeofenceConfig

__all__ = ["RouteStepsResponse", "Completion", "Meta"]


class Completion(BaseModel):
    completed_at: Optional[int] = None
    """Represents the timestamp of the completion in seconds since the Unix epoch.

    Example: `1738743999`.
    """

    completed_by_mode: Optional[RouteStepCompletionMode] = None
    """Specify the mode of completion to be used for the step.

    Currently, following values are allowed:

    - `manual`: Steps must be marked as completed manually through the Driver App.
    - `geofence`: Steps are marked as completed automatically based on the entry
      conditions and geofence specified.
    - `geofence_manual_fallback`: Steps will be marked as completed automatically
      based on geofence and entry condition configurations but there will also be a
      provision for manually updating the status in case, geofence detection fails.
    """

    completion_mode: Optional[RouteStepCompletionMode] = None
    """Specify the mode of completion to be used for the step.

    Currently, following values are allowed:

    - `manual`: Steps must be marked as completed manually through the Driver App.
    - `geofence`: Steps are marked as completed automatically based on the entry
      conditions and geofence specified.
    - `geofence_manual_fallback`: Steps will be marked as completed automatically
      based on geofence and entry condition configurations but there will also be a
      provision for manually updating the status in case, geofence detection fails.
    """

    document: Optional[object] = None
    """
    A key-value map storing form submission data, where keys correspond to field
    labels and values can be of any type depend on the type of according document
    item.
    """

    document_modified_at: Optional[int] = None
    """
    Represents the timestamp of the last doc modification in seconds since the Unix
    epoch. Example: `1738743999`.
    """

    geofence_config: Optional[RouteStepGeofenceConfig] = None
    """
    Specify the configurations of the geofence which will be used to detect presence
    of the driver and complete the tasks automatically. Please note that this
    attribute is required when `completion_mode` is either "geofence" or
    "geofence_manual_fallback".
    """

    status: Optional[Literal["scheduled", "completed", "canceled"]] = None
    """Status of the step."""


class Meta(BaseModel):
    customer_name: Optional[str] = None
    """Returns the customer name associated with the step.

    It can configured in the input request using the `metadata` attribute of the
    step.
    """

    customer_phone_number: Optional[str] = None
    """Returns the customer's phone number associated with the step.

    It can configured in the input request using the `metadata` attribute of the
    step.
    """

    instructions: Optional[str] = None
    """Returns the custom instructions to carry out while performing the task.

    These instructions can be provided at the time of configuring the step details
    in the input request.
    """


class RouteStepsResponse(BaseModel):
    id: Optional[str] = None
    """Returns the unique ID of the step."""

    address: Optional[str] = None
    """Returns the postal address where the step is executed.

    Its value is the same as that specified in the input request when configuring
    the step details.
    """

    arrival: Optional[int] = None
    """
    Returns the scheduled arrival time of the driver at the step as an UNIX
    timestamp, in seconds precision. It is the same as that specified in the input
    request while configuring the step details.

    The timestamp returned here is only for informative display on the driver's app
    and it does not impact or get affected by the route generated.
    """

    completion: Optional[Completion] = None

    created_at: Optional[int] = None
    """Represents the timestamp of the creation in seconds since the Unix epoch.

    Example: `1738743999`.
    """

    document_snapshot: Optional[List[object]] = None
    """
    Returns the details of the document that was used for collecting the proof of
    completion for the step. In case no document template ID was provided for the
    given step, then a `null` value is returned. Each object represents a new field
    in the document.
    """

    duration: Optional[int] = None
    """Returns the duration for `layover` or `break` type steps."""

    location: Optional[List[float]] = None
    """Returns the location coordinates where the step is executed."""

    meta: Optional[Meta] = None
    """
    An object returning custom details about the step that were configured in the
    input request while configuring the step details. The information returned here
    will be available for display on the Driver's app under step details.
    """

    short_id: Optional[str] = None
    """
    Returns a unique short ID of the step for easier referencing and displaying
    purposes.
    """

    type: Optional[str] = None
    """Returns the step type.

    It can belong to one of the following: `start`, `job` , `pickup`, `delivery`,
    `break`, `layover` , and `end`. For any given step, it would be the same as that
    specified in the input request while configuring the step details.
    """

    updated_at: Optional[int] = None
    """Represents the timestamp of the last update in seconds since the Unix epoch.

    Example: `1738743999`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import discover_retrieve_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.discover_retrieve_response import DiscoverRetrieveResponse

__all__ = ["DiscoverResource", "AsyncDiscoverResource"]


class DiscoverResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DiscoverResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DiscoverResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DiscoverResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return DiscoverResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        key: str,
        q: str,
        at: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DiscoverRetrieveResponse:
        """
        Discover matching places

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          q: Specify the free-text search query.

              Please note that whitespace, urls, email addresses, or other out-of-scope
              queries will yield no results.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a country (or multiple countries), provided as comma-separated
                [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
                codes

                The country codes are to be provided in all uppercase.

                Format: `countryCode:{countryCode}[,{countryCode}]`

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          limit: Sets the maximum number of results to be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/discover",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "q": q,
                        "at": at,
                        "in_": in_,
                        "lang": lang,
                        "limit": limit,
                    },
                    discover_retrieve_params.DiscoverRetrieveParams,
                ),
            ),
            cast_to=DiscoverRetrieveResponse,
        )


class AsyncDiscoverResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDiscoverResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDiscoverResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDiscoverResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncDiscoverResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        key: str,
        q: str,
        at: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DiscoverRetrieveResponse:
        """
        Discover matching places

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          q: Specify the free-text search query.

              Please note that whitespace, urls, email addresses, or other out-of-scope
              queries will yield no results.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a country (or multiple countries), provided as comma-separated
                [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
                codes

                The country codes are to be provided in all uppercase.

                Format: `countryCode:{countryCode}[,{countryCode}]`

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          limit: Sets the maximum number of results to be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/discover",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "q": q,
                        "at": at,
                        "in_": in_,
                        "lang": lang,
                        "limit": limit,
                    },
                    discover_retrieve_params.DiscoverRetrieveParams,
                ),
            ),
            cast_to=DiscoverRetrieveResponse,
        )


class DiscoverResourceWithRawResponse:
    def __init__(self, discover: DiscoverResource) -> None:
        self._discover = discover

        self.retrieve = to_raw_response_wrapper(
            discover.retrieve,
        )


class AsyncDiscoverResourceWithRawResponse:
    def __init__(self, discover: AsyncDiscoverResource) -> None:
        self._discover = discover

        self.retrieve = async_to_raw_response_wrapper(
            discover.retrieve,
        )


class DiscoverResourceWithStreamingResponse:
    def __init__(self, discover: DiscoverResource) -> None:
        self._discover = discover

        self.retrieve = to_streamed_response_wrapper(
            discover.retrieve,
        )


class AsyncDiscoverResourceWithStreamingResponse:
    def __init__(self, discover: AsyncDiscoverResource) -> None:
        self._discover = discover

        self.retrieve = async_to_streamed_response_wrapper(
            discover.retrieve,
        )
